(* This file was generated automatically by wayland-scanner-ocaml *)

[@@@ocaml.warning "-27"]
open struct
  module Imports = struct
    include Linux_dmabuf_unstable_v1_proto
    include Wayland.Wayland_proto
  end
  
  module Proxy = Wayland.Proxy
  module Msg = Wayland.Msg
  module Fixed = Wayland.Fixed
  module Iface_reg = Wayland.Iface_reg
  module S = Wayland.S
end


(** Factory for creating dmabuf-based wl_buffers.
    
    Following the interfaces from:
    https://www.khronos.org/registry/egl/extensions/EXT/EGL_EXT_image_dma_buf_import.txt
    https://www.khronos.org/registry/EGL/extensions/EXT/EGL_EXT_image_dma_buf_import_modifiers.txt
    and the Linux DRM sub-system's AddFb2 ioctl.
    
    This interface offers ways to create generic dmabuf-based
    wl_buffers. Immediately after a client binds to this interface,
    the set of supported formats and format modifiers is sent with
    'format' and 'modifier' events.
    
    The following are required from clients:
    
    - Clients must ensure that either all data in the dma-buf is
    coherent for all subsequent read access or that coherency is
    correctly handled by the underlying kernel-side dma-buf
    implementation.
    
    - Don't make any more attachments after sending the buffer to the
    compositor. Making more attachments later increases the risk of
    the compositor not being able to use (re-import) an existing
    dmabuf-based wl_buffer.
    
    The underlying graphics stack must ensure the following:
    
    - The dmabuf file descriptors relayed to the server will stay valid
    for the whole lifetime of the wl_buffer. This means the server may
    at any time use those fds to import the dmabuf into any kernel
    sub-system that might accept it.
    
    However, when the underlying graphics stack fails to deliver the
    promise, because of e.g. a device hot-unplug which raises internal
    errors, after the wl_buffer has been successfully created the
    compositor must not raise protocol errors to the client when dmabuf
    import later fails.
    
    To create a wl_buffer from one or more dmabufs, a client creates a
    zwp_linux_dmabuf_params_v1 object with a zwp_linux_dmabuf_v1.create_params
    request. All planes required by the intended format are added with
    the 'add' request. Finally, a 'create' or 'create_immed' request is
    issued, which has the following outcome depending on the import success.
    
    The 'create' request,
    - on success, triggers a 'created' event which provides the final
    wl_buffer to the client.
    - on failure, triggers a 'failed' event to convey that the server
    cannot use the dmabufs received from the client.
    
    For the 'create_immed' request,
    - on success, the server immediately imports the added dmabufs to
    create a wl_buffer. No event is sent from the server in this case.
    - on failure, the server can choose to either:
    - terminate the client by raising a fatal error.
    - mark the wl_buffer as failed, and send a 'failed' event to the
    client. If the client uses a failed wl_buffer as an argument to any
    request, the behaviour is compositor implementation-defined.
    
    Warning! The protocol described in this file is experimental and
    backward incompatible changes may be made. Backward compatible changes
    may be added together with the corresponding interface version bump.
    Backward incompatible changes are done by bumping the version number in
    the protocol and interface names and resetting the interface version.
    Once the protocol is to be declared stable, the 'z' prefix and the
    version number in the protocol and interface names are removed and the
    interface version number is reset. *)
module Zwp_linux_dmabuf_v1 = struct
  type 'v t = ([`Zwp_linux_dmabuf_v1], 'v, [`Server]) Proxy.t
  
  (** {2 Version 1, 2} *)
  
  (** Supported buffer format.
      
      This event advertises one buffer format that the server supports.
      All the supported formats are advertised once when the client
      binds to this interface. A roundtrip after binding guarantees
      that the client has received all supported formats.
      
      For the definition of the format codes, see the
      zwp_linux_buffer_params_v1::create request.
      
      Warning: the 'format' event is likely to be deprecated and replaced
      with the 'modifier' event introduced in zwp_linux_dmabuf_v1
      version 3, described below. Please refrain from using the information
      received from this event. *)
  let format (_t:([< `V1 | `V2 | `V3] as 'v) t) ~format =
    let _msg = Proxy.alloc _t ~op:0 ~ints:1 ~strings:[] ~arrays:[] in
    Msg.add_int _msg format;
    Proxy.send _t _msg
  
  
  (** {2 Version 3} *)
  
  (** Supported buffer format modifier.
      
      This event advertises the formats that the server supports, along with
      the modifiers supported for each format. All the supported modifiers
      for all the supported formats are advertised once when the client
      binds to this interface. A roundtrip after binding guarantees that
      the client has received all supported format-modifier pairs.
      
      For legacy support, DRM_FORMAT_MOD_INVALID (that is, modifier_hi ==
      0x00ffffff and modifier_lo == 0xffffffff) is allowed in this event.
      It indicates that the server can support the format with an implicit
      modifier. When a plane has DRM_FORMAT_MOD_INVALID as its modifier, it
      is as if no explicit modifier is specified. The effective modifier
      will be derived from the dmabuf.
      
      A compositor that sends valid modifiers and DRM_FORMAT_MOD_INVALID for
      a given format supports both explicit modifiers and implicit modifiers.
      
      For the definition of the format and modifier codes, see the
      zwp_linux_buffer_params_v1::create and zwp_linux_buffer_params_v1::add
      requests. *)
  let modifier (_t:([< `V3] as 'v) t) ~format ~modifier_hi ~modifier_lo =
    let _msg = Proxy.alloc _t ~op:1 ~ints:3 ~strings:[] ~arrays:[] in
    Msg.add_int _msg format;
    Msg.add_int _msg modifier_hi;
    Msg.add_int _msg modifier_lo;
    Proxy.send _t _msg
  
  (**/**)
  class virtual ['v] _handlers_unsafe = object (_self : (_, 'v, _) #Proxy.Handler.t)
    method user_data = S.No_data
    method metadata = (module Linux_dmabuf_unstable_v1_proto.Zwp_linux_dmabuf_v1)
    method max_version = 3l
    
    method private virtual on_destroy : [> ] t -> unit
    
    method private virtual on_create_params : [> ] t -> ([`Zwp_linux_buffer_params_v1], 'v, [`Server]) Proxy.t -> unit
    
    
    method dispatch (_proxy : 'v t) _msg =
      let _proxy = Proxy.cast_version _proxy in
      match Msg.op _msg with
      | 0 ->
        Proxy.shutdown_recv _proxy;
        _self#on_destroy _proxy 
      | 1 ->
        let params_id : ([`Zwp_linux_buffer_params_v1], _, _) Proxy.t =
          Msg.get_int _msg |> Proxy.Handler.accept_new _proxy (module Imports.Zwp_linux_buffer_params_v1) in
        _self#on_create_params _proxy params_id
      | _ -> assert false
  end
  (**/**)
  
  (** {2 Handlers}
      Note: Servers will always want to use [v1].
   *)
  
  
  (** Handler for a proxy with version >= 1. *)
  class virtual ['v] v1 = object (_ : (_, 'v, _) #Proxy.Service_handler.t)
    (**/**)
    inherit [[< `V1 | `V2 | `V3] as 'v] _handlers_unsafe
    (**/**)
    method private virtual on_destroy : [> `V1 | `V2 | `V3] t -> unit
    
    (** Unbind the factory.
        
        Objects created through this interface, especially wl_buffers, will
        remain valid. *)
    
    method private virtual on_create_params : [> `V1 | `V2 | `V3] t -> ([`Zwp_linux_buffer_params_v1], 'v, [`Server]) Proxy.t ->
                                              unit
    
    (** Create a temporary object for buffer parameters.
        
        This temporary object is used to collect multiple dmabuf handles into
        a single batch to create a wl_buffer. It can only be used once and
        should be destroyed after a 'created' or 'failed' event has been
        received. *)
    
    method min_version = 1l
    method bind_version : [`V1] = `V1
  end
  
  (** Handler for a proxy with version >= 2. *)
  class virtual ['v] v2 = object (_ : (_, 'v, _) #Proxy.Service_handler.t)
    (**/**)
    inherit [[< `V2 | `V3] as 'v] _handlers_unsafe
    (**/**)
    method private virtual on_destroy : [> `V2 | `V3] t -> unit
    
    (** Unbind the factory.
        
        Objects created through this interface, especially wl_buffers, will
        remain valid. *)
    
    method private virtual on_create_params : [> `V2 | `V3] t -> ([`Zwp_linux_buffer_params_v1], 'v, [`Server]) Proxy.t ->
                                              unit
    
    (** Create a temporary object for buffer parameters.
        
        This temporary object is used to collect multiple dmabuf handles into
        a single batch to create a wl_buffer. It can only be used once and
        should be destroyed after a 'created' or 'failed' event has been
        received. *)
    
    method min_version = 2l
    method bind_version : [`V2] = `V2
  end
  
  (** Handler for a proxy with version >= 3. *)
  class virtual ['v] v3 = object (_ : (_, 'v, _) #Proxy.Service_handler.t)
    (**/**)
    inherit [[< `V3] as 'v] _handlers_unsafe
    (**/**)
    method private virtual on_destroy : [> `V3] t -> unit
    
    (** Unbind the factory.
        
        Objects created through this interface, especially wl_buffers, will
        remain valid. *)
    
    method private virtual on_create_params : [> `V3] t -> ([`Zwp_linux_buffer_params_v1], 'v, [`Server]) Proxy.t ->
                                              unit
    
    (** Create a temporary object for buffer parameters.
        
        This temporary object is used to collect multiple dmabuf handles into
        a single batch to create a wl_buffer. It can only be used once and
        should be destroyed after a 'created' or 'failed' event has been
        received. *)
    
    method min_version = 3l
    method bind_version : [`V3] = `V3
  end
end

(** Parameters for creating a dmabuf-based wl_buffer.
    
    This temporary object is a collection of dmabufs and other
    parameters that together form a single logical buffer. The temporary
    object may eventually create one wl_buffer unless cancelled by
    destroying it before requesting 'create'.
    
    Single-planar formats only require one dmabuf, however
    multi-planar formats may require more than one dmabuf. For all
    formats, an 'add' request must be called once per plane (even if the
    underlying dmabuf fd is identical).
    
    You must use consecutive plane indices ('plane_idx' argument for 'add')
    from zero to the number of planes used by the drm_fourcc format code.
    All planes required by the format must be given exactly once, but can
    be given in any order. Each plane index can be set only once. *)
module Zwp_linux_buffer_params_v1 = struct
  type 'v t = ([`Zwp_linux_buffer_params_v1], 'v, [`Server]) Proxy.t
  module Error = Linux_dmabuf_unstable_v1_proto.Zwp_linux_buffer_params_v1.Error
  
  module Flags = Linux_dmabuf_unstable_v1_proto.Zwp_linux_buffer_params_v1.Flags
  
  (** {2 Version 1} *)
  
  (** Buffer creation failed.
      
      This event indicates that the attempted buffer creation has
      failed. It usually means that one of the dmabuf constraints
      has not been fulfilled.
      
      Upon receiving this event, the client should destroy the
      zlinux_buffer_params object. *)
  let failed (_t:([< `V1 | `V2 | `V3] as 'v) t)  =
    let _msg = Proxy.alloc _t ~op:1 ~ints:0 ~strings:[] ~arrays:[] in
    Proxy.send _t _msg
  
  (** Buffer creation succeeded.
      
      This event indicates that the attempted buffer creation was
      successful. It provides the new wl_buffer referencing the dmabuf(s).
      
      Upon receiving this event, the client should destroy the
      zlinux_dmabuf_params object. *)
  let created (_t:([< `V1 | `V2 | `V3] as 'v) t) (buffer:([`Wl_buffer], 'v, [`Server]) #Proxy.Handler.t) =
    let __buffer = Proxy.spawn _t buffer in
    let _msg = Proxy.alloc _t ~op:0 ~ints:1 ~strings:[] ~arrays:[] in
    Msg.add_int _msg (Proxy.id __buffer);
    Proxy.send _t _msg;
    __buffer
  
  
  (** {2 Version 2, 3} *)
  
  (**/**)
  class virtual ['v] _handlers_unsafe = object (_self : (_, 'v, _) #Proxy.Handler.t)
    method user_data = S.No_data
    method metadata = (module Linux_dmabuf_unstable_v1_proto.Zwp_linux_buffer_params_v1)
    method max_version = 3l
    
    method private virtual on_destroy : [> ] t -> unit
    
    method private virtual on_add : [> ] t -> fd:Unix.file_descr -> plane_idx:int32 -> offset:int32 -> stride:int32 ->
                                    modifier_hi:int32 -> modifier_lo:int32 -> unit
    
    method private virtual on_create : [> ] t -> width:int32 -> height:int32 -> format:int32 ->
                                       flags:Imports.Zwp_linux_buffer_params_v1.Flags.t -> unit
    
    method private virtual on_create_immed : [> ] t -> ([`Wl_buffer], 'v, [`Server]) Proxy.t -> width:int32 ->
                                             height:int32 -> format:int32 ->
                                             flags:Imports.Zwp_linux_buffer_params_v1.Flags.t -> unit
    
    
    method dispatch (_proxy : 'v t) _msg =
      let _proxy = Proxy.cast_version _proxy in
      match Msg.op _msg with
      | 0 ->
        Proxy.shutdown_recv _proxy;
        _self#on_destroy _proxy 
      | 1 ->
        let fd = Msg.get_fd _msg in
        let plane_idx = Msg.get_int _msg in
        let offset = Msg.get_int _msg in
        let stride = Msg.get_int _msg in
        let modifier_hi = Msg.get_int _msg in
        let modifier_lo = Msg.get_int _msg in
        _self#on_add _proxy ~fd ~plane_idx ~offset ~stride ~modifier_hi ~modifier_lo
      | 2 ->
        let width = Msg.get_int _msg in
        let height = Msg.get_int _msg in
        let format = Msg.get_int _msg in
        let flags = Msg.get_int _msg |> Imports.Zwp_linux_buffer_params_v1.Flags.of_int32 in
        _self#on_create _proxy ~width ~height ~format ~flags
      | 3 ->
        let buffer_id : ([`Wl_buffer], _, _) Proxy.t =
          Msg.get_int _msg |> Proxy.Handler.accept_new _proxy (module Imports.Wl_buffer) in
        let width = Msg.get_int _msg in
        let height = Msg.get_int _msg in
        let format = Msg.get_int _msg in
        let flags = Msg.get_int _msg |> Imports.Zwp_linux_buffer_params_v1.Flags.of_int32 in
        _self#on_create_immed _proxy buffer_id ~width ~height ~format ~flags
      | _ -> assert false
  end
  (**/**)
  
  (** {2 Handlers}
      Note: Servers will always want to use [v1].
   *)
  
  
  (** Handler for a proxy with version >= 1. *)
  class virtual ['v] v1 = object (_ : (_, 'v, _) #Proxy.Service_handler.t)
    (**/**)
    inherit [[< `V1 | `V2 | `V3] as 'v] _handlers_unsafe
    (**/**)
    method private virtual on_destroy : [> `V1 | `V2 | `V3] t -> unit
    
    (** Delete this object, used or not.
        
        Cleans up the temporary data sent to the server for dmabuf-based
        wl_buffer creation. *)
    
    method private virtual on_add : [> `V1 | `V2 | `V3] t -> fd:Unix.file_descr -> plane_idx:int32 -> offset:int32 ->
                                    stride:int32 -> modifier_hi:int32 -> modifier_lo:int32 -> unit
    
    (** Add a dmabuf to the temporary set.
        
        This request adds one dmabuf to the set in this
        zwp_linux_buffer_params_v1.
        
        The 64-bit unsigned value combined from modifier_hi and modifier_lo
        is the dmabuf layout modifier. DRM AddFB2 ioctl calls this the
        fb modifier, which is defined in drm_mode.h of Linux UAPI.
        This is an opaque token. Drivers use this token to express tiling,
        compression, etc. driver-specific modifications to the base format
        defined by the DRM fourcc code.
        
        Warning: It should be an error if the format/modifier pair was not
        advertised with the modifier event. This is not enforced yet because
        some implementations always accept DRM_FORMAT_MOD_INVALID. Also
        version 2 of this protocol does not have the modifier event.
        
        This request raises the PLANE_IDX error if plane_idx is too large.
        The error PLANE_SET is raised if attempting to set a plane that
        was already set. *)
    
    method private virtual on_create : [> `V1 | `V2 | `V3] t -> width:int32 -> height:int32 -> format:int32 ->
                                       flags:Imports.Zwp_linux_buffer_params_v1.Flags.t -> unit
    
    (** Create a wl_buffer from the given dmabufs.
        
        This asks for creation of a wl_buffer from the added dmabuf
        buffers. The wl_buffer is not created immediately but returned via
        the 'created' event if the dmabuf sharing succeeds. The sharing
        may fail at runtime for reasons a client cannot predict, in
        which case the 'failed' event is triggered.
        
        The 'format' argument is a DRM_FORMAT code, as defined by the
        libdrm's drm_fourcc.h. The Linux kernel's DRM sub-system is the
        authoritative source on how the format codes should work.
        
        The 'flags' is a bitfield of the flags defined in enum "flags".
        'y_invert' means the that the image needs to be y-flipped.
        
        Flag 'interlaced' means that the frame in the buffer is not
        progressive as usual, but interlaced. An interlaced buffer as
        supported here must always contain both top and bottom fields.
        The top field always begins on the first pixel row. The temporal
        ordering between the two fields is top field first, unless
        'bottom_first' is specified. It is undefined whether 'bottom_first'
        is ignored if 'interlaced' is not set.
        
        This protocol does not convey any information about field rate,
        duration, or timing, other than the relative ordering between the
        two fields in one buffer. A compositor may have to estimate the
        intended field rate from the incoming buffer rate. It is undefined
        whether the time of receiving wl_surface.commit with a new buffer
        attached, applying the wl_surface state, wl_surface.frame callback
        trigger, presentation, or any other point in the compositor cycle
        is used to measure the frame or field times. There is no support
        for detecting missed or late frames/fields/buffers either, and
        there is no support whatsoever for cooperating with interlaced
        compositor output.
        
        The composited image quality resulting from the use of interlaced
        buffers is explicitly undefined. A compositor may use elaborate
        hardware features or software to deinterlace and create progressive
        output frames from a sequence of interlaced input buffers, or it
        may produce substandard image quality. However, compositors that
        cannot guarantee reasonable image quality in all cases are recommended
        to just reject all interlaced buffers.
        
        Any argument errors, including non-positive width or height,
        mismatch between the number of planes and the format, bad
        format, bad offset or stride, may be indicated by fatal protocol
        errors: INCOMPLETE, INVALID_FORMAT, INVALID_DIMENSIONS,
        OUT_OF_BOUNDS.
        
        Dmabuf import errors in the server that are not obvious client
        bugs are returned via the 'failed' event as non-fatal. This
        allows attempting dmabuf sharing and falling back in the client
        if it fails.
        
        This request can be sent only once in the object's lifetime, after
        which the only legal request is destroy. This object should be
        destroyed after issuing a 'create' request. Attempting to use this
        object after issuing 'create' raises ALREADY_USED protocol error.
        
        It is not mandatory to issue 'create'. If a client wants to
        cancel the buffer creation, it can just destroy this object. *)
    
    method private virtual on_create_immed : [> `V2 | `V3] t -> ([`Wl_buffer], 'v, [`Server]) Proxy.t -> width:int32 ->
                                             height:int32 -> format:int32 ->
                                             flags:Imports.Zwp_linux_buffer_params_v1.Flags.t -> unit
    
    (** Immediately create a wl_buffer from the given dmabufs.
        
        This asks for immediate creation of a wl_buffer by importing the
        added dmabufs.
        
        In case of import success, no event is sent from the server, and the
        wl_buffer is ready to be used by the client.
        
        Upon import failure, either of the following may happen, as seen fit
        by the implementation:
        - the client is terminated with one of the following fatal protocol
        errors:
        - INCOMPLETE, INVALID_FORMAT, INVALID_DIMENSIONS, OUT_OF_BOUNDS,
        in case of argument errors such as mismatch between the number
        of planes and the format, bad format, non-positive width or
        height, or bad offset or stride.
        - INVALID_WL_BUFFER, in case the cause for failure is unknown or
        plaform specific.
        - the server creates an invalid wl_buffer, marks it as failed and
        sends a 'failed' event to the client. The result of using this
        invalid wl_buffer as an argument in any request by the client is
        defined by the compositor implementation.
        
        This takes the same arguments as a 'create' request, and obeys the
        same restrictions. *)
    
    method min_version = 1l
  end
  
  (** Handler for a proxy with version >= 2. *)
  class virtual ['v] v2 = object (_ : (_, 'v, _) #Proxy.Service_handler.t)
    (**/**)
    inherit [[< `V2 | `V3] as 'v] _handlers_unsafe
    (**/**)
    method private virtual on_destroy : [> `V2 | `V3] t -> unit
    
    (** Delete this object, used or not.
        
        Cleans up the temporary data sent to the server for dmabuf-based
        wl_buffer creation. *)
    
    method private virtual on_add : [> `V2 | `V3] t -> fd:Unix.file_descr -> plane_idx:int32 -> offset:int32 ->
                                    stride:int32 -> modifier_hi:int32 -> modifier_lo:int32 -> unit
    
    (** Add a dmabuf to the temporary set.
        
        This request adds one dmabuf to the set in this
        zwp_linux_buffer_params_v1.
        
        The 64-bit unsigned value combined from modifier_hi and modifier_lo
        is the dmabuf layout modifier. DRM AddFB2 ioctl calls this the
        fb modifier, which is defined in drm_mode.h of Linux UAPI.
        This is an opaque token. Drivers use this token to express tiling,
        compression, etc. driver-specific modifications to the base format
        defined by the DRM fourcc code.
        
        Warning: It should be an error if the format/modifier pair was not
        advertised with the modifier event. This is not enforced yet because
        some implementations always accept DRM_FORMAT_MOD_INVALID. Also
        version 2 of this protocol does not have the modifier event.
        
        This request raises the PLANE_IDX error if plane_idx is too large.
        The error PLANE_SET is raised if attempting to set a plane that
        was already set. *)
    
    method private virtual on_create : [> `V2 | `V3] t -> width:int32 -> height:int32 -> format:int32 ->
                                       flags:Imports.Zwp_linux_buffer_params_v1.Flags.t -> unit
    
    (** Create a wl_buffer from the given dmabufs.
        
        This asks for creation of a wl_buffer from the added dmabuf
        buffers. The wl_buffer is not created immediately but returned via
        the 'created' event if the dmabuf sharing succeeds. The sharing
        may fail at runtime for reasons a client cannot predict, in
        which case the 'failed' event is triggered.
        
        The 'format' argument is a DRM_FORMAT code, as defined by the
        libdrm's drm_fourcc.h. The Linux kernel's DRM sub-system is the
        authoritative source on how the format codes should work.
        
        The 'flags' is a bitfield of the flags defined in enum "flags".
        'y_invert' means the that the image needs to be y-flipped.
        
        Flag 'interlaced' means that the frame in the buffer is not
        progressive as usual, but interlaced. An interlaced buffer as
        supported here must always contain both top and bottom fields.
        The top field always begins on the first pixel row. The temporal
        ordering between the two fields is top field first, unless
        'bottom_first' is specified. It is undefined whether 'bottom_first'
        is ignored if 'interlaced' is not set.
        
        This protocol does not convey any information about field rate,
        duration, or timing, other than the relative ordering between the
        two fields in one buffer. A compositor may have to estimate the
        intended field rate from the incoming buffer rate. It is undefined
        whether the time of receiving wl_surface.commit with a new buffer
        attached, applying the wl_surface state, wl_surface.frame callback
        trigger, presentation, or any other point in the compositor cycle
        is used to measure the frame or field times. There is no support
        for detecting missed or late frames/fields/buffers either, and
        there is no support whatsoever for cooperating with interlaced
        compositor output.
        
        The composited image quality resulting from the use of interlaced
        buffers is explicitly undefined. A compositor may use elaborate
        hardware features or software to deinterlace and create progressive
        output frames from a sequence of interlaced input buffers, or it
        may produce substandard image quality. However, compositors that
        cannot guarantee reasonable image quality in all cases are recommended
        to just reject all interlaced buffers.
        
        Any argument errors, including non-positive width or height,
        mismatch between the number of planes and the format, bad
        format, bad offset or stride, may be indicated by fatal protocol
        errors: INCOMPLETE, INVALID_FORMAT, INVALID_DIMENSIONS,
        OUT_OF_BOUNDS.
        
        Dmabuf import errors in the server that are not obvious client
        bugs are returned via the 'failed' event as non-fatal. This
        allows attempting dmabuf sharing and falling back in the client
        if it fails.
        
        This request can be sent only once in the object's lifetime, after
        which the only legal request is destroy. This object should be
        destroyed after issuing a 'create' request. Attempting to use this
        object after issuing 'create' raises ALREADY_USED protocol error.
        
        It is not mandatory to issue 'create'. If a client wants to
        cancel the buffer creation, it can just destroy this object. *)
    
    method private virtual on_create_immed : [> `V2 | `V3] t -> ([`Wl_buffer], 'v, [`Server]) Proxy.t -> width:int32 ->
                                             height:int32 -> format:int32 ->
                                             flags:Imports.Zwp_linux_buffer_params_v1.Flags.t -> unit
    
    (** Immediately create a wl_buffer from the given dmabufs.
        
        This asks for immediate creation of a wl_buffer by importing the
        added dmabufs.
        
        In case of import success, no event is sent from the server, and the
        wl_buffer is ready to be used by the client.
        
        Upon import failure, either of the following may happen, as seen fit
        by the implementation:
        - the client is terminated with one of the following fatal protocol
        errors:
        - INCOMPLETE, INVALID_FORMAT, INVALID_DIMENSIONS, OUT_OF_BOUNDS,
        in case of argument errors such as mismatch between the number
        of planes and the format, bad format, non-positive width or
        height, or bad offset or stride.
        - INVALID_WL_BUFFER, in case the cause for failure is unknown or
        plaform specific.
        - the server creates an invalid wl_buffer, marks it as failed and
        sends a 'failed' event to the client. The result of using this
        invalid wl_buffer as an argument in any request by the client is
        defined by the compositor implementation.
        
        This takes the same arguments as a 'create' request, and obeys the
        same restrictions. *)
    
    method min_version = 2l
  end
  
  (** Handler for a proxy with version >= 3. *)
  class virtual ['v] v3 = object (_ : (_, 'v, _) #Proxy.Service_handler.t)
    (**/**)
    inherit [[< `V3] as 'v] _handlers_unsafe
    (**/**)
    method private virtual on_destroy : [> `V3] t -> unit
    
    (** Delete this object, used or not.
        
        Cleans up the temporary data sent to the server for dmabuf-based
        wl_buffer creation. *)
    
    method private virtual on_add : [> `V3] t -> fd:Unix.file_descr -> plane_idx:int32 -> offset:int32 ->
                                    stride:int32 -> modifier_hi:int32 -> modifier_lo:int32 -> unit
    
    (** Add a dmabuf to the temporary set.
        
        This request adds one dmabuf to the set in this
        zwp_linux_buffer_params_v1.
        
        The 64-bit unsigned value combined from modifier_hi and modifier_lo
        is the dmabuf layout modifier. DRM AddFB2 ioctl calls this the
        fb modifier, which is defined in drm_mode.h of Linux UAPI.
        This is an opaque token. Drivers use this token to express tiling,
        compression, etc. driver-specific modifications to the base format
        defined by the DRM fourcc code.
        
        Warning: It should be an error if the format/modifier pair was not
        advertised with the modifier event. This is not enforced yet because
        some implementations always accept DRM_FORMAT_MOD_INVALID. Also
        version 2 of this protocol does not have the modifier event.
        
        This request raises the PLANE_IDX error if plane_idx is too large.
        The error PLANE_SET is raised if attempting to set a plane that
        was already set. *)
    
    method private virtual on_create : [> `V3] t -> width:int32 -> height:int32 -> format:int32 ->
                                       flags:Imports.Zwp_linux_buffer_params_v1.Flags.t -> unit
    
    (** Create a wl_buffer from the given dmabufs.
        
        This asks for creation of a wl_buffer from the added dmabuf
        buffers. The wl_buffer is not created immediately but returned via
        the 'created' event if the dmabuf sharing succeeds. The sharing
        may fail at runtime for reasons a client cannot predict, in
        which case the 'failed' event is triggered.
        
        The 'format' argument is a DRM_FORMAT code, as defined by the
        libdrm's drm_fourcc.h. The Linux kernel's DRM sub-system is the
        authoritative source on how the format codes should work.
        
        The 'flags' is a bitfield of the flags defined in enum "flags".
        'y_invert' means the that the image needs to be y-flipped.
        
        Flag 'interlaced' means that the frame in the buffer is not
        progressive as usual, but interlaced. An interlaced buffer as
        supported here must always contain both top and bottom fields.
        The top field always begins on the first pixel row. The temporal
        ordering between the two fields is top field first, unless
        'bottom_first' is specified. It is undefined whether 'bottom_first'
        is ignored if 'interlaced' is not set.
        
        This protocol does not convey any information about field rate,
        duration, or timing, other than the relative ordering between the
        two fields in one buffer. A compositor may have to estimate the
        intended field rate from the incoming buffer rate. It is undefined
        whether the time of receiving wl_surface.commit with a new buffer
        attached, applying the wl_surface state, wl_surface.frame callback
        trigger, presentation, or any other point in the compositor cycle
        is used to measure the frame or field times. There is no support
        for detecting missed or late frames/fields/buffers either, and
        there is no support whatsoever for cooperating with interlaced
        compositor output.
        
        The composited image quality resulting from the use of interlaced
        buffers is explicitly undefined. A compositor may use elaborate
        hardware features or software to deinterlace and create progressive
        output frames from a sequence of interlaced input buffers, or it
        may produce substandard image quality. However, compositors that
        cannot guarantee reasonable image quality in all cases are recommended
        to just reject all interlaced buffers.
        
        Any argument errors, including non-positive width or height,
        mismatch between the number of planes and the format, bad
        format, bad offset or stride, may be indicated by fatal protocol
        errors: INCOMPLETE, INVALID_FORMAT, INVALID_DIMENSIONS,
        OUT_OF_BOUNDS.
        
        Dmabuf import errors in the server that are not obvious client
        bugs are returned via the 'failed' event as non-fatal. This
        allows attempting dmabuf sharing and falling back in the client
        if it fails.
        
        This request can be sent only once in the object's lifetime, after
        which the only legal request is destroy. This object should be
        destroyed after issuing a 'create' request. Attempting to use this
        object after issuing 'create' raises ALREADY_USED protocol error.
        
        It is not mandatory to issue 'create'. If a client wants to
        cancel the buffer creation, it can just destroy this object. *)
    
    method private virtual on_create_immed : [> `V3] t -> ([`Wl_buffer], 'v, [`Server]) Proxy.t -> width:int32 ->
                                             height:int32 -> format:int32 ->
                                             flags:Imports.Zwp_linux_buffer_params_v1.Flags.t -> unit
    
    (** Immediately create a wl_buffer from the given dmabufs.
        
        This asks for immediate creation of a wl_buffer by importing the
        added dmabufs.
        
        In case of import success, no event is sent from the server, and the
        wl_buffer is ready to be used by the client.
        
        Upon import failure, either of the following may happen, as seen fit
        by the implementation:
        - the client is terminated with one of the following fatal protocol
        errors:
        - INCOMPLETE, INVALID_FORMAT, INVALID_DIMENSIONS, OUT_OF_BOUNDS,
        in case of argument errors such as mismatch between the number
        of planes and the format, bad format, non-positive width or
        height, or bad offset or stride.
        - INVALID_WL_BUFFER, in case the cause for failure is unknown or
        plaform specific.
        - the server creates an invalid wl_buffer, marks it as failed and
        sends a 'failed' event to the client. The result of using this
        invalid wl_buffer as an argument in any request by the client is
        defined by the compositor implementation.
        
        This takes the same arguments as a 'create' request, and obeys the
        same restrictions. *)
    
    method min_version = 3l
  end
end