(* This file was generated automatically by wayland-scanner-ocaml *)

[@@@ocaml.warning "-27"]
open struct
  module Imports = struct
    include Xdg_decoration_unstable_v1_proto
    include Wayland.Wayland_proto
    include Xdg_shell_proto
  end
  
  module Proxy = Wayland.Proxy
  module Msg = Wayland.Msg
  module Fixed = Wayland.Fixed
  module Iface_reg = Wayland.Iface_reg
  module S = Wayland.S
end


(** Window decoration manager.
    
    This interface allows a compositor to announce support for server-side
    decorations.
    
    A window decoration is a set of window controls as deemed appropriate by
    the party managing them, such as user interface components used to move,
    resize and change a window's state.
    
    A client can use this protocol to request being decorated by a supporting
    compositor.
    
    If compositor and client do not negotiate the use of a server-side
    decoration using this protocol, clients continue to self-decorate as they
    see fit.
    
    Warning! The protocol described in this file is experimental and
    backward incompatible changes may be made. Backward compatible changes
    may be added together with the corresponding interface version bump.
    Backward incompatible changes are done by bumping the version number in
    the protocol and interface names and resetting the interface version.
    Once the protocol is to be declared stable, the 'z' prefix and the
    version number in the protocol and interface names are removed and the
    interface version number is reset. *)
module Zxdg_decoration_manager_v1 = struct
  type 'v t = ([`Zxdg_decoration_manager_v1], 'v, [`Client]) Proxy.t
  
  (** {2 Version 1} *)
  
  (** Create a new toplevel decoration object.
      
      Create a new decoration object associated with the given toplevel.
      
      Creating an xdg_toplevel_decoration from an xdg_toplevel which has a
      buffer attached or committed is a client error, and any attempts by a
      client to attach or manipulate a buffer prior to the first
      xdg_toplevel_decoration.configure event must also be treated as
      errors. *)
  let get_toplevel_decoration (_t:([< `V1] as 'v) t) (id:([`Zxdg_toplevel_decoration_v1], 'v, [`Client]) #Proxy.Handler.t) ~(toplevel:([`Xdg_toplevel], _, [`Client]) Proxy.t) =
    let __id = Proxy.spawn _t id in
    let _msg = Proxy.alloc _t ~op:1 ~ints:2 ~strings:[] ~arrays:[] in
    Msg.add_int _msg (Proxy.id __id);
    Msg.add_int _msg (Proxy.id toplevel);
    Proxy.send _t _msg;
    __id
  
  (** Destroy the decoration manager object.
      
      Destroy the decoration manager. This doesn't destroy objects created
      with the manager. *)
  let destroy (_t:([< `V1] as 'v) t)  =
    let _msg = Proxy.alloc _t ~op:0 ~ints:0 ~strings:[] ~arrays:[] in
    Proxy.send _t _msg;
    Proxy.shutdown_send _t
  
  (**/**)
  class ['v] _handlers_unsafe = object (_self : (_, 'v, _) #Proxy.Handler.t)
    method user_data = S.No_data
    method metadata = (module Xdg_decoration_unstable_v1_proto.Zxdg_decoration_manager_v1)
    method max_version = 1l
    
    
    method dispatch (_proxy : 'v t) _msg =
      let _proxy = Proxy.cast_version _proxy in
      match Msg.op _msg with
      | _ -> assert false
  end
  (**/**)
  
  (** {2 Handlers}
      Note: Servers will always want to use [v1].
   *)
  
  
  (** Handler for a proxy with version >= 1. *)
  class ['v] v1 = object (_ : (_, 'v, _) #Proxy.Service_handler.t)
    (**/**)
    inherit [[< `V1] as 'v] _handlers_unsafe
    (**/**)
    method min_version = 1l
    method bind_version : [`V1] = `V1
  end
end

(** Decoration object for a toplevel surface.
    
    The decoration object allows the compositor to toggle server-side window
    decorations for a toplevel surface. The client can request to switch to
    another mode.
    
    The xdg_toplevel_decoration object must be destroyed before its
    xdg_toplevel. *)
module Zxdg_toplevel_decoration_v1 = struct
  type 'v t = ([`Zxdg_toplevel_decoration_v1], 'v, [`Client]) Proxy.t
  module Error = Xdg_decoration_unstable_v1_proto.Zxdg_toplevel_decoration_v1.Error
  
  module Mode = Xdg_decoration_unstable_v1_proto.Zxdg_toplevel_decoration_v1.Mode
  
  (** {2 Version 1} *)
  
  (** Unset the decoration mode.
      
      Unset the toplevel surface decoration mode. This informs the compositor
      that the client doesn't prefer a particular decoration mode.
      
      This request has the same semantics as set_mode. *)
  let unset_mode (_t:([< `V1] as 'v) t)  =
    let _msg = Proxy.alloc _t ~op:2 ~ints:0 ~strings:[] ~arrays:[] in
    Proxy.send _t _msg
  
  (** Set the decoration mode.
      
      Set the toplevel surface decoration mode. This informs the compositor
      that the client prefers the provided decoration mode.
      
      After requesting a decoration mode, the compositor will respond by
      emitting an xdg_surface.configure event. The client should then update
      its content, drawing it without decorations if the received mode is
      server-side decorations. The client must also acknowledge the configure
      when committing the new content (see xdg_surface.ack_configure).
      
      The compositor can decide not to use the client's mode and enforce a
      different mode instead.
      
      Clients whose decoration mode depend on the xdg_toplevel state may send
      a set_mode request in response to an xdg_surface.configure event and wait
      for the next xdg_surface.configure event to prevent unwanted state.
      Such clients are responsible for preventing configure loops and must
      make sure not to send multiple successive set_mode requests with the
      same decoration mode. *)
  let set_mode (_t:([< `V1] as 'v) t) ~mode =
    let _msg = Proxy.alloc _t ~op:1 ~ints:1 ~strings:[] ~arrays:[] in
    Msg.add_int _msg (Imports.Zxdg_toplevel_decoration_v1.Mode.to_int32 mode);
    Proxy.send _t _msg
  
  (** Destroy the decoration object.
      
      Switch back to a mode without any server-side decorations at the next
      commit. *)
  let destroy (_t:([< `V1] as 'v) t)  =
    let _msg = Proxy.alloc _t ~op:0 ~ints:0 ~strings:[] ~arrays:[] in
    Proxy.send _t _msg;
    Proxy.shutdown_send _t
  
  (**/**)
  class virtual ['v] _handlers_unsafe = object (_self : (_, 'v, _) #Proxy.Handler.t)
    method user_data = S.No_data
    method metadata = (module Xdg_decoration_unstable_v1_proto.Zxdg_toplevel_decoration_v1)
    method max_version = 1l
    
    method private virtual on_configure : [> ] t -> mode:Imports.Zxdg_toplevel_decoration_v1.Mode.t -> unit
    
    
    method dispatch (_proxy : 'v t) _msg =
      let _proxy = Proxy.cast_version _proxy in
      match Msg.op _msg with
      | 0 ->
        let mode = Msg.get_int _msg |> Imports.Zxdg_toplevel_decoration_v1.Mode.of_int32 in
        _self#on_configure _proxy ~mode
      | _ -> assert false
  end
  (**/**)
  
  (** {2 Handlers}
      Note: Servers will always want to use [v1].
   *)
  
  
  (** Handler for a proxy with version >= 1. *)
  class virtual ['v] v1 = object (_ : (_, 'v, _) #Proxy.Service_handler.t)
    (**/**)
    inherit [[< `V1] as 'v] _handlers_unsafe
    (**/**)
    method private virtual on_configure : [> `V1] t -> mode:Imports.Zxdg_toplevel_decoration_v1.Mode.t -> unit
    
    (** Suggest a surface change.
        
        The configure event asks the client to change its decoration mode. The
        configured state should not be applied immediately. Clients must send an
        ack_configure in response to this event. See xdg_surface.configure and
        xdg_surface.ack_configure for details.
        
        A configure event can be sent at any time. The specified mode must be
        obeyed by the client. *)
    
    method min_version = 1l
  end
end